/*
 *  Copyright (c) 2020 Agata Cacko <cacko.azh@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "TestKoZoomAction.h"

#include <QTest>
#include <QDialog>

#include <QVBoxLayout>

#include "kis_debug.h"
#include "KoZoomMode.h"

void TestKoZoomAction::test()
{
    KoZoomAction* action = new KoZoomAction(KoZoomMode::ZOOM_CONSTANT, "zoom", 0);
    qreal prev, after;
    do {
        prev = action->effectiveZoom();
        action->zoomIn();
        after = action->effectiveZoom();
        //qDebug() << prev << after;
    } while (after != prev);
    //qDebug() << action->maximumZoom();
    //qDebug() << action->effectiveZoom();
    //qDebug() << action->nextZoomLevel();

    QVERIFY(action->nextZoomLevel() == action->effectiveZoom());
    delete action;
}


QTEST_MAIN(TestKoZoomAction)
